# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/atnclib',
                'doc_host': 'https://atncai.github.io',
                'git_url': 'https://github.com/atncai/atnclib',
                'lib_path': 'atnclib'},
  'syms': { 'atnclib.callbacks': { 'atnclib.callbacks.Callback': ('callbacks.html#callback', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.DeviceCB': ('callbacks.html#devicecb', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.DeviceCB.__init__': ('callbacks.html#devicecb.__init__', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.DeviceCB.before_batch': ( 'callbacks.html#devicecb.before_batch',
                                                                                'atnclib/callbacks.py'),
                                   'atnclib.callbacks.DeviceCB.before_fit': ('callbacks.html#devicecb.before_fit', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB': ('callbacks.html#memorycb', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.after_backprop': ( 'callbacks.html#memorycb.after_backprop',
                                                                                  'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.after_batch': ( 'callbacks.html#memorycb.after_batch',
                                                                               'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.after_epoch': ( 'callbacks.html#memorycb.after_epoch',
                                                                               'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.after_loss': ('callbacks.html#memorycb.after_loss', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.after_preds': ( 'callbacks.html#memorycb.after_preds',
                                                                               'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.before_batch': ( 'callbacks.html#memorycb.before_batch',
                                                                                'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.before_epoch': ( 'callbacks.html#memorycb.before_epoch',
                                                                                'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.before_fit': ('callbacks.html#memorycb.before_fit', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MemoryCB.before_preds': ( 'callbacks.html#memorycb.before_preds',
                                                                                'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB': ('callbacks.html#metricscb', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB.__init__': ('callbacks.html#metricscb.__init__', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB._log': ('callbacks.html#metricscb._log', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB.after_batch': ( 'callbacks.html#metricscb.after_batch',
                                                                                'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB.after_epoch': ( 'callbacks.html#metricscb.after_epoch',
                                                                                'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB.after_fit': ('callbacks.html#metricscb.after_fit', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB.before_epoch': ( 'callbacks.html#metricscb.before_epoch',
                                                                                 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.MetricsCB.before_fit': ( 'callbacks.html#metricscb.before_fit',
                                                                               'atnclib/callbacks.py'),
                                   'atnclib.callbacks.SaveModelCB': ('callbacks.html#savemodelcb', 'atnclib/callbacks.py'),
                                   'atnclib.callbacks.SaveModelCB.__init__': ( 'callbacks.html#savemodelcb.__init__',
                                                                               'atnclib/callbacks.py'),
                                   'atnclib.callbacks.SaveModelCB.after_epoch': ( 'callbacks.html#savemodelcb.after_epoch',
                                                                                  'atnclib/callbacks.py'),
                                   'atnclib.callbacks.get_mem': ('callbacks.html#get_mem', 'atnclib/callbacks.py')},
            'atnclib.core': {'atnclib.core.foo': ('core-copy1.html#foo', 'atnclib/core.py')},
            'atnclib.data': { 'atnclib.data.ATNCDataset': ('data.html#atncdataset', 'atnclib/data.py'),
                              'atnclib.data.ATNCDataset.__getitem__': ('data.html#atncdataset.__getitem__', 'atnclib/data.py'),
                              'atnclib.data.ATNCDataset.__init__': ('data.html#atncdataset.__init__', 'atnclib/data.py'),
                              'atnclib.data.ATNCDataset.__len__': ('data.html#atncdataset.__len__', 'atnclib/data.py'),
                              'atnclib.data.ATNCDataset.get_mask_fns': ('data.html#atncdataset.get_mask_fns', 'atnclib/data.py'),
                              'atnclib.data.DataLoaders': ('data.html#dataloaders', 'atnclib/data.py'),
                              'atnclib.data.DataLoaders.__init__': ('data.html#dataloaders.__init__', 'atnclib/data.py'),
                              'atnclib.data.get_augmentation': ('data.html#get_augmentation', 'atnclib/data.py'),
                              'atnclib.data.get_dataloaders': ('data.html#get_dataloaders', 'atnclib/data.py'),
                              'atnclib.data.get_dataset_fns': ('data.html#get_dataset_fns', 'atnclib/data.py'),
                              'atnclib.data.get_datasets': ('data.html#get_datasets', 'atnclib/data.py'),
                              'atnclib.data.get_preprocessing': ('data.html#get_preprocessing', 'atnclib/data.py'),
                              'atnclib.data.preproc_img': ('data.html#preproc_img', 'atnclib/data.py')},
            'atnclib.learner': { 'atnclib.learner.CancelBatchException': ('learner.html#cancelbatchexception', 'atnclib/learner.py'),
                                 'atnclib.learner.CancelEpochException': ('learner.html#cancelepochexception', 'atnclib/learner.py'),
                                 'atnclib.learner.CancelFitException': ('learner.html#cancelfitexception', 'atnclib/learner.py'),
                                 'atnclib.learner.Learner': ('learner.html#learner', 'atnclib/learner.py'),
                                 'atnclib.learner.Learner.__init__': ('learner.html#learner.__init__', 'atnclib/learner.py'),
                                 'atnclib.learner.Learner.callback': ('learner.html#learner.callback', 'atnclib/learner.py'),
                                 'atnclib.learner.Learner.fit': ('learner.html#learner.fit', 'atnclib/learner.py'),
                                 'atnclib.learner.Learner.one_batch': ('learner.html#learner.one_batch', 'atnclib/learner.py'),
                                 'atnclib.learner.Learner.one_epoch': ('learner.html#learner.one_epoch', 'atnclib/learner.py'),
                                 'atnclib.learner.run_cbs': ('learner.html#run_cbs', 'atnclib/learner.py')},
            'atnclib.losses': { 'atnclib.losses.CombinedLoss': ('losses.html#combinedloss', 'atnclib/losses.py'),
                                'atnclib.losses.CombinedLoss.__call__': ('losses.html#combinedloss.__call__', 'atnclib/losses.py'),
                                'atnclib.losses.CombinedLoss.__init__': ('losses.html#combinedloss.__init__', 'atnclib/losses.py'),
                                'atnclib.losses.CombinedLoss.activation': ('losses.html#combinedloss.activation', 'atnclib/losses.py'),
                                'atnclib.losses.CombinedLoss.decodes': ('losses.html#combinedloss.decodes', 'atnclib/losses.py')},
            'atnclib.metrics': { 'atnclib.metrics.Accuracy': ('metrics.html#accuracy', 'atnclib/metrics.py'),
                                 'atnclib.metrics.Accuracy.__init__': ('metrics.html#accuracy.__init__', 'atnclib/metrics.py'),
                                 'atnclib.metrics.Accuracy.update': ('metrics.html#accuracy.update', 'atnclib/metrics.py'),
                                 'atnclib.metrics.IOU_Score': ('metrics.html#iou_score', 'atnclib/metrics.py'),
                                 'atnclib.metrics.IOU_Score.__init__': ('metrics.html#iou_score.__init__', 'atnclib/metrics.py'),
                                 'atnclib.metrics.IOU_Score.update': ('metrics.html#iou_score.update', 'atnclib/metrics.py'),
                                 'atnclib.metrics.Recall': ('metrics.html#recall', 'atnclib/metrics.py'),
                                 'atnclib.metrics.Recall.__init__': ('metrics.html#recall.__init__', 'atnclib/metrics.py'),
                                 'atnclib.metrics.Recall.update': ('metrics.html#recall.update', 'atnclib/metrics.py'),
                                 'atnclib.metrics.SegmentationMetric': ('metrics.html#segmentationmetric', 'atnclib/metrics.py'),
                                 'atnclib.metrics.SegmentationMetric.__init__': ( 'metrics.html#segmentationmetric.__init__',
                                                                                  'atnclib/metrics.py'),
                                 'atnclib.metrics.SegmentationMetric.compute': ( 'metrics.html#segmentationmetric.compute',
                                                                                 'atnclib/metrics.py'),
                                 'atnclib.metrics.SegmentationMetric.merge_state': ( 'metrics.html#segmentationmetric.merge_state',
                                                                                     'atnclib/metrics.py')},
            'atnclib.model': { 'atnclib.model.calc_confusion_matrix': ('model.html#calc_confusion_matrix', 'atnclib/model.py'),
                               'atnclib.model.create_checkpoint_data': ('model.html#create_checkpoint_data', 'atnclib/model.py'),
                               'atnclib.model.display_confusion_matrix': ('model.html#display_confusion_matrix', 'atnclib/model.py'),
                               'atnclib.model.get_model_name': ('model.html#get_model_name', 'atnclib/model.py'),
                               'atnclib.model.get_preprocessiong_fn': ('model.html#get_preprocessiong_fn', 'atnclib/model.py'),
                               'atnclib.model.get_seg_model': ('model.html#get_seg_model', 'atnclib/model.py'),
                               'atnclib.model.restore_model': ('model.html#restore_model', 'atnclib/model.py'),
                               'atnclib.model.save_model_checkpoint': ('model.html#save_model_checkpoint', 'atnclib/model.py')},
            'atnclib.utility': { 'atnclib.utility.expand_to_32px': ('utility.html#expand_to_32px', 'atnclib/utility.py'),
                                 'atnclib.utility.load_cls_labels': ('utility.html#load_cls_labels', 'atnclib/utility.py'),
                                 'atnclib.utility.mask_code_to_continues_indices': ( 'utility.html#mask_code_to_continues_indices',
                                                                                     'atnclib/utility.py'),
                                 'atnclib.utility.mask_continues_indices_to_codes': ( 'utility.html#mask_continues_indices_to_codes',
                                                                                      'atnclib/utility.py'),
                                 'atnclib.utility.mask_continues_indices_to_idx': ( 'utility.html#mask_continues_indices_to_idx',
                                                                                    'atnclib/utility.py'),
                                 'atnclib.utility.mask_continues_indices_to_rgb': ( 'utility.html#mask_continues_indices_to_rgb',
                                                                                    'atnclib/utility.py'),
                                 'atnclib.utility.mask_map_classes': ('utility.html#mask_map_classes', 'atnclib/utility.py'),
                                 'atnclib.utility.mask_to_rgb': ('utility.html#mask_to_rgb', 'atnclib/utility.py'),
                                 'atnclib.utility.one_hot_encode': ('utility.html#one_hot_encode', 'atnclib/utility.py'),
                                 'atnclib.utility.reverse_one_hot': ('utility.html#reverse_one_hot', 'atnclib/utility.py'),
                                 'atnclib.utility.to_cpu': ('utility.html#to_cpu', 'atnclib/utility.py'),
                                 'atnclib.utility.to_device': ('utility.html#to_device', 'atnclib/utility.py'),
                                 'atnclib.utility.to_tensor': ('utility.html#to_tensor', 'atnclib/utility.py')},
            'atnclib.visualization': { 'atnclib.visualization.convert_image': ( 'visualization.html#convert_image',
                                                                                'atnclib/visualization.py'),
                                       'atnclib.visualization.show_image_row': ( 'visualization.html#show_image_row',
                                                                                 'atnclib/visualization.py')}}}
